@@ -67,6 +67,17 @@ find_groups1(Mod, GrNames, TCs, GroupDefs) ->
 		 GroupDefs, FindAll),
     [Conf || Conf <- Found, Conf /= 'NOMATCH'].
 
+%% Finds out if the given group, or child cases / child group of a group, is non-empty.
+not_empty_conf({conf, _Props, _Init, Children, _End}) -> not_empty_conf(Children);
+not_empty_conf([] = _Cases) -> false;
+not_empty_conf([_ | _] = _Cases) -> true;
+not_empty_conf(Name) when is_atom(Name) -> true;
+% Other, unexpected formats we don't actively care about, such as `{mnesia_evil_backup, all}`.
+not_empty_conf(_Format) -> true.
+
+remove_empty_confs(Confs) ->
+    lists:filtermap(fun not_empty_conf/1, Confs).
+
 %% Locate all groups
 find(Mod, all, all, [{Name,Props,Tests} | Gs], Known, Defs, _) 
   when is_atom(Name), is_list(Props), is_list(Tests) ->
@@ -254,7 +265,6 @@ find(_Mod, _GrNames, _TCs, [], _Known, _Defs, _) ->
 
 trim({conf,Props,Init,Tests,End}) ->
     try trim(Tests) of
-	[] -> [];
 	Tests1 -> [{conf,Props,Init,Tests1,End}]
     catch
 	throw:_ -> []
@@ -500,7 +510,7 @@ make_conf(Mod, Name, Props, TestSpec) ->
 %%%-----------------------------------------------------------------
 
 expand_groups([H | T], ConfTests, Mod) ->
-    [expand_groups(H, ConfTests, Mod) | expand_groups(T, ConfTests, Mod)];
+    remove_empty_confs([expand_groups(H, ConfTests, Mod) | expand_groups(T, ConfTests, Mod)]);
 expand_groups([], _ConfTests, _Mod) ->
     [];
 expand_groups({group,Name}, ConfTests, Mod) ->
