@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2024. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -32,7 +32,8 @@
          validate_signature/6,
          verify_data/1,
          verify_fun/4,
-         prepare_for_next_cert/2]).
+         prepare_for_next_cert/2,
+         apply_fun/5]).
 
 %% Utility functions
 -export([normalize_general_name/1,
@@ -65,13 +66,13 @@
 %%====================================================================
 
 %%--------------------------------------------------------------------
--spec init_validation_state(#'OTPCertificate'{}, integer(), list()) ->
+-spec init_validation_state(#cert{}, integer(), list()) ->
 				   #path_validation_state{}.
 %%
 %% Description: Creates initial version of path_validation_state for
 %% basic path validation of x509 certificates.
 %%--------------------------------------------------------------------
-init_validation_state(#'OTPCertificate'{} = OtpCert, DefaultPathLen,
+init_validation_state(Cert, DefaultPathLen,
 		      Options) ->
     PolicyTree = pubkey_policy_tree:root(),
     MaxLen =  proplists:get_value(max_path_length, Options, DefaultPathLen),
@@ -96,33 +97,34 @@ init_validation_state(#'OTPCertificate'{} = OtpCert, DefaultPathLen,
 				   verify_fun              = VerifyFun,
 				   user_state	           = UserState,
 				   cert_num                = 0},
-    prepare_for_next_cert(OtpCert, State).
+    prepare_for_next_cert(Cert, State).
 
 %%--------------------------------------------------------------------
--spec validate_extensions(#'OTPCertificate'{}, #path_validation_state{},
+-spec validate_extensions(#cert{}, #path_validation_state{},
 			  term(), fun())->
 				 {#path_validation_state{}, UserState :: term()}.
 %%
 %% Description: Check extensions included in basic path validation.
 %%--------------------------------------------------------------------
-validate_extensions(OtpCert, ValidationState0, UserState0, VerifyFun) ->
+validate_extensions(Cert, ValidationState0, UserState0, VerifyFun) ->
+    OtpCert = otp_cert(Cert),
     TBSCert = OtpCert#'OTPCertificate'.tbsCertificate,
     case TBSCert#'OTPTBSCertificate'.version of
 	N when N >= 3 ->
 	    Extensions = TBSCert#'OTPTBSCertificate'.extensions,
-            validate_extensions(OtpCert, Extensions,
+            validate_extensions(Cert, Extensions,
                                 ValidationState0, no_basic_constraint,
                                 is_self_signed(OtpCert), UserState0, VerifyFun);
         _ -> %% Extensions not present in versions 1 & 2
 	    {ValidationState0, UserState0}
     end.
 %%--------------------------------------------------------------------
--spec validate_policy_tree(#'OTPCertificate'{}, #path_validation_state{})->
+-spec validate_policy_tree(#cert{}, #path_validation_state{})->
           #path_validation_state{} | no_return().
 %%
 %% Description: Check policy tree requirements after handling of certificate extensions
 %%--------------------------------------------------------------------
-validate_policy_tree(OtpCert,
+validate_policy_tree(Cert,
                      #path_validation_state{explicit_policy = ExplicitPolicyConstraint,
                                             valid_policy_tree = Tree,
                                             user_state = UserState0,
@@ -133,9 +135,9 @@ validate_policy_tree(OtpCert,
             ValidationState;
         false ->
             UserState =
-                verify_fun(OtpCert, {bad_cert,
-                                     {policy_requirement_not_met,
-                                      {{explicit_policy, ExplicitPolicyConstraint},
+                verify_fun(Cert, {bad_cert,
+                                  {policy_requirement_not_met,
+                                   {{explicit_policy, ExplicitPolicyConstraint},
                                        {policy_set,
                                         pubkey_policy_tree:constrained_policy_node_set(Tree)}}}},
                            UserState0, VerifyFun),
@@ -143,21 +145,22 @@ validate_policy_tree(OtpCert,
     end.
 
 %%--------------------------------------------------------------------
--spec validate_time(#'OTPCertificate'{}, term(), fun()) -> term().
+-spec validate_time(#cert{}, term(), fun()) -> term().
 %%
 %% Description: Check that the certificate validity period includes the
 %% current time.
 %%--------------------------------------------------------------------
-validate_time(OtpCert, UserState, VerifyFun) ->
+validate_time(Cert, UserState, VerifyFun) ->
     % Parse and check validity of the certificate dates, and if it fails, invoke `verify_fun` to
     % hand over control to the caller in order to decide what to do.
+    OtpCert = otp_cert(Cert),
     case parse_and_check_validity_dates(OtpCert) of
         expired ->
             % Certificate has correctly formatted dates but it's expired
-            verify_fun(OtpCert, {bad_cert, cert_expired}, UserState, VerifyFun);
+            verify_fun(Cert, {bad_cert, cert_expired}, UserState, VerifyFun);
         error ->
             % Certificate has incorrectly formatted dates, attempt to delegate decision to app function
-            verify_fun(OtpCert, {bad_cert, invalid_validity_dates}, UserState, VerifyFun);
+            verify_fun(Cert, {bad_cert, invalid_validity_dates}, UserState, VerifyFun);
         % Validation succeded and certificate is not expired, no new state needed
         ok ->
             UserState
@@ -191,26 +194,28 @@ parse_and_check_validity_dates(OtpCert) ->
     end.
 
 %%--------------------------------------------------------------------
--spec validate_issuer(#'OTPCertificate'{}, term(), term(), fun()) -> term() | no_return().
+-spec validate_issuer(#cert{}, term(), term(), fun()) -> term() | no_return().
 %%
 %% Description: Check that the certificate issuer name is the working_issuer_name
 %% in path_validation_state.
 %%--------------------------------------------------------------------
-validate_issuer(OtpCert, Issuer, UserState, VerifyFun) ->
+validate_issuer(Cert, Issuer, UserState, VerifyFun) ->
+    OtpCert = otp_cert(Cert),
     TBSCert = OtpCert#'OTPCertificate'.tbsCertificate,
     case is_issuer(Issuer, TBSCert#'OTPTBSCertificate'.issuer) of
 	true ->
 	    UserState;
 	_ ->
-	    verify_fun(OtpCert, {bad_cert, invalid_issuer}, UserState, VerifyFun)
+	    verify_fun(Cert, {bad_cert, invalid_issuer}, UserState, VerifyFun)
     end.
 %%--------------------------------------------------------------------
--spec validate_names(#'OTPCertificate'{}, no_constraints | list(), list(),
+-spec validate_names(#cert{}, no_constraints | list(), list(),
 		     term(), term(), fun())-> term() | no_return().
 %%
 %% Description: Validate Subject Alternative Name.
 %%--------------------------------------------------------------------
-validate_names(OtpCert, Permit, Exclude, Last, UserState, VerifyFun) ->
+validate_names(Cert, Permit, Exclude, Last, UserState, VerifyFun) ->
+    OtpCert = otp_cert(Cert),
     case is_self_signed(OtpCert) andalso (not Last) of
 	true ->
 	    UserState;
@@ -239,13 +244,13 @@ validate_names(OtpCert, Permit, Exclude, Last, UserState, VerifyFun) ->
 		true ->
 		    UserState;
 		false ->
-		    verify_fun(OtpCert, {bad_cert, name_not_permitted},
+		    verify_fun(Cert, {bad_cert, name_not_permitted},
 			      UserState, VerifyFun)
 	    end
     end.
 
 %%--------------------------------------------------------------------
--spec validate_signature(#'OTPCertificate'{}, DER::binary(),
+-spec validate_signature(#cert{}, DER::binary(),
 			 term(),term(), term(), fun()) -> term() | no_return().
 
 %%
@@ -253,14 +258,15 @@ validate_names(OtpCert, Permit, Exclude, Last, UserState, VerifyFun) ->
 %% working_public_key_algorithm, the working_public_key, and
 %% the working_public_key_parameters in path_validation_state.
 %%--------------------------------------------------------------------
-validate_signature(OtpCert, DerCert, Key, KeyParams,
+validate_signature(Cert, DerCert, Key, KeyParams0,
 		   UserState, VerifyFun) ->
-
+    OtpCert = otp_cert(Cert),
+    KeyParams = key_params(OtpCert#'OTPCertificate'.tbsCertificate, KeyParams0),
     case verify_signature(OtpCert, DerCert, Key, KeyParams) of
 	true ->
 	    UserState;
 	false ->
-	    verify_fun(OtpCert, {bad_cert, invalid_signature}, UserState, VerifyFun)
+	    verify_fun(Cert, {bad_cert, invalid_signature}, UserState, VerifyFun)
     end.
 
 %%--------------------------------------------------------------------
@@ -277,7 +283,7 @@ verify_data(DerCert) ->
     extract_verify_data(OtpCert, DerCert).
 
 %%--------------------------------------------------------------------
--spec verify_fun(#'OTPCertificate'{}, {bad_cert, public_key:bad_cert_reason()} |
+-spec verify_fun(#cert{}, {bad_cert, public_key:bad_cert_reason()} |
                  {extension, #'Extension'{}}|
 		 valid | valid_peer, term(), fun()) -> term() | no_return().
 %%
@@ -285,9 +291,9 @@ verify_data(DerCert) ->
 %% validation errors and unknown extensions and optional do other
 %% things with a validated certificate.
 %% --------------------------------------------------------------------
-verify_fun(Otpcert, Result, UserState0, VerifyFun) ->
-    case VerifyFun(Otpcert, Result, UserState0) of
-	{valid, UserState} ->
+verify_fun(#cert{der = DerCert, otp = OtpCert}, Result, UserState0, VerifyFun) ->
+    case apply_fun(VerifyFun, OtpCert, DerCert, Result, UserState0) of
+        {valid, UserState} ->
 	    UserState;
 	{valid_peer, UserState} ->
 	    UserState;
@@ -308,21 +314,21 @@ verify_fun(Otpcert, Result, UserState0, VerifyFun) ->
     end.
 
 %%--------------------------------------------------------------------
--spec prepare_for_next_cert(#'OTPCertificate'{}, #path_validation_state{}) ->
+-spec prepare_for_next_cert(#cert{}, #path_validation_state{}) ->
 				   #path_validation_state{} | no_return().
 %%
 %% Description: Update path_validation_state for next iteration.
 %%--------------------------------------------------------------------
-prepare_for_next_cert(OtpCert, #path_validation_state{policy_mapping_ext = Ext} =
+prepare_for_next_cert(Cert, #path_validation_state{policy_mapping_ext = Ext} =
                           ValidationState0) when Ext =/= undefined ->
-    ValidationState1 = handle_policy_mappings(OtpCert, ValidationState0),
+    ValidationState1 = handle_policy_mappings(Cert, ValidationState0),
     ValidationState =
         ValidationState1#path_validation_state{policy_mapping_ext =
                                                    undefined,
                                                current_any_policy_qualifiers =
                                                    undefined},
-    prepare_for_next_cert(OtpCert, ValidationState);
-prepare_for_next_cert(OtpCert, #path_validation_state{
+    prepare_for_next_cert(Cert, ValidationState);
+prepare_for_next_cert(Cert, #path_validation_state{
                                   working_public_key_algorithm = PrevAlgo,
                                   working_public_key_parameters =
                                       PrevParams,
@@ -331,6 +337,7 @@ prepare_for_next_cert(OtpCert, #path_validation_state{
                                   inhibit_policy_mapping = PolicyMappingConstraint,
                                   inhibit_any_policy = AnyPolicyConstraint
                                  } = ValidationState0) ->
+    OtpCert = otp_cert(Cert),
     TBSCert = OtpCert#'OTPCertificate'.tbsCertificate,
     Issuer =  TBSCert#'OTPTBSCertificate'.subject,
 
@@ -372,7 +379,15 @@ prepare_for_next_cert(OtpCert, #path_validation_state{
          },
     ValidationState2 = handle_policy_constraints(ValidationState1),
     ValidationState = handle_inhibit_anypolicy(ValidationState2),
-    handle_last_cert(OtpCert, ValidationState).
+    handle_last_cert(Cert, ValidationState).
+
+apply_fun(Fun, OtpCert, DerCert, Result, UserState) ->
+    if is_function(Fun, 4) ->
+            Fun(OtpCert, DerCert, Result, UserState);
+       is_function(Fun, 3) ->
+            Fun(OtpCert, Result, UserState)
+    end.
+
 
 %%====================================================================
 %% Utility functions
@@ -518,10 +533,21 @@ match_name(emailAddress, Name, [PermittedName | Rest]) ->
 
 match_name(dNSName, Name, [PermittedName | Rest]) ->
     Fun = fun(Domain, [$.|Domain]) -> true;
-	     (Name1,Name2) ->
-		  is_suffix(Name2, Name1)
+	     (Name1, [$. | _] = Name2) ->
+                  is_suffix(Name2, Name1);
+             (Name1, Name2) ->
+                  StrLen1 = string:len(Name1),
+                  StrLen2 = string:len(Name2),
+                  case StrLen1 > StrLen2 of
+                      true ->
+                          is_suffix([$. | Name2], Name1);
+                      false when StrLen1 == StrLen2 ->
+                          string:casefold(Name1) == string:casefold(Name2);
+                      false ->
+                          false
+                  end
           end,
-    match_name(Fun, Name, [$.|PermittedName], Rest);
+    match_name(Fun, Name, PermittedName, Rest);
 
 match_name(x400Address, OrAddress, [PermittedAddr | Rest]) ->
     match_name(fun is_or_address/2, OrAddress, PermittedAddr, Rest);
@@ -671,15 +697,16 @@ root_cert(Name, Opts) ->
 %%--------------------------------------------------------------------
 
 %% No extensions present
-validate_extensions(OtpCert, asn1_NOVALUE, ValidationState, ExistBasicCon,
+validate_extensions(Cert, asn1_NOVALUE, ValidationState, ExistBasicCon,
 		    SelfSigned, UserState, VerifyFun) ->
-    validate_extensions(OtpCert, [], ValidationState, ExistBasicCon,
+    validate_extensions(Cert, [], ValidationState, ExistBasicCon,
 			SelfSigned, UserState, VerifyFun);
 
-validate_extensions(_,[], ValidationState, basic_constraint, _SelfSigned,
-		    UserState, _) ->
+validate_extensions(Cert, [], ValidationState, basic_constraint, _SelfSigned,
+		    UserState0, VerifyFun) ->
+    UserState = validate_ext_key_usage(Cert, UserState0, VerifyFun, ca),
     {ValidationState, UserState};
-validate_extensions(OtpCert, [], ValidationState =
+validate_extensions(Cert, [], ValidationState =
 			#path_validation_state{max_path_length = Len,
 					       last_cert = Last},
 		    no_basic_constraint, SelfSigned, UserState0, VerifyFun) ->
@@ -687,22 +714,22 @@ validate_extensions(OtpCert, [], ValidationState =
 	true when SelfSigned ->
 	    {ValidationState, UserState0};
 	true  ->
+            UserState = validate_ext_key_usage(Cert, UserState0, VerifyFun, endentity),
 	    {ValidationState#path_validation_state{max_path_length = Len - 1},
-	     UserState0};
+	     UserState};
 	false ->
 	    %% basic_constraint must appear in certs used for digital sign
 	    %% see 4.2.1.10 in rfc 3280
-	    case is_digitally_sign_cert(OtpCert) of
+	    case is_digitally_sign_cert(Cert) of
 		true ->
-		    missing_basic_constraints(OtpCert, SelfSigned,
+		    missing_basic_constraints(Cert, SelfSigned,
 					      ValidationState, VerifyFun,
 					      UserState0, Len);
 		false -> %% Example CRL signer only
 		    {ValidationState, UserState0}
 	    end
     end;
-
-validate_extensions(OtpCert,
+validate_extensions(Cert,
 		    [#'Extension'{extnID = ?'id-ce-basicConstraints',
 				  extnValue =
 				      #'BasicConstraints'{cA = true,
@@ -714,55 +741,55 @@ validate_extensions(OtpCert,
     Length = if SelfSigned -> erlang:min(N, Len);
 		true -> erlang:min(N, Len-1)
 	     end,
-    validate_extensions(OtpCert, Rest,
+    validate_extensions(Cert, Rest,
 			ValidationState#path_validation_state{max_path_length =
 								  Length},
 			basic_constraint, SelfSigned,
 			UserState, VerifyFun);
 %% The pathLenConstraint field is meaningful only if cA is set to
 %% TRUE.
-validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-basicConstraints',
+validate_extensions(Cert, [#'Extension'{extnID = ?'id-ce-basicConstraints',
 					   extnValue =
 					       #'BasicConstraints'{cA = false}} |
 			      Rest], ValidationState, ExistBasicCon,
 		    SelfSigned, UserState, VerifyFun) ->
-    validate_extensions(OtpCert, Rest, ValidationState, ExistBasicCon,
+    validate_extensions(Cert, Rest, ValidationState, ExistBasicCon,
 			SelfSigned, UserState, VerifyFun);
 
-validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-keyUsage',
-					   extnValue = KeyUse
-					  } | Rest],
+validate_extensions(Cert, [#'Extension'{extnID = ?'id-ce-keyUsage',
+                                        extnValue = KeyUses
+                                       } | Rest],
 		    #path_validation_state{last_cert=Last} = ValidationState,
 		    ExistBasicCon, SelfSigned,
 		    UserState0, VerifyFun) ->
-    case Last orelse is_valid_key_usage(KeyUse, keyCertSign) of
+    case Last orelse lists:member(keyCertSign, KeyUses) of
 	true ->
-	    validate_extensions(OtpCert, Rest, ValidationState, ExistBasicCon,
+	    validate_extensions(Cert, Rest, ValidationState, ExistBasicCon,
 				SelfSigned, UserState0, VerifyFun);
 	false ->
-	    UserState = verify_fun(OtpCert, {bad_cert, invalid_key_usage},
+	    UserState = verify_fun(Cert, {bad_cert, invalid_key_usage},
 				   UserState0, VerifyFun),
-	    validate_extensions(OtpCert, Rest, ValidationState, ExistBasicCon,
+	    validate_extensions(Cert, Rest, ValidationState, ExistBasicCon,
 				SelfSigned, UserState, VerifyFun)
     end;
 
-validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-subjectAltName',
+validate_extensions(Cert, [#'Extension'{extnID = ?'id-ce-subjectAltName',
 					   extnValue = Names,
 					   critical = true} = Ext | Rest],
 		    ValidationState, ExistBasicCon,
 		    SelfSigned, UserState0, VerifyFun)  ->
     case validate_subject_alt_names(Names) of
 	true  ->
-	    validate_extensions(OtpCert, Rest, ValidationState, ExistBasicCon,
+	    validate_extensions(Cert, Rest, ValidationState, ExistBasicCon,
 				SelfSigned, UserState0, VerifyFun);
 	false ->
-	    UserState = verify_fun(OtpCert, {extension, Ext},
+	    UserState = verify_fun(Cert, {extension, Ext},
 				   UserState0, VerifyFun),
-	    validate_extensions(OtpCert, Rest, ValidationState, ExistBasicCon,
+	    validate_extensions(Cert, Rest, ValidationState, ExistBasicCon,
 				SelfSigned, UserState, VerifyFun)
     end;
 
-validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-nameConstraints',
+validate_extensions(Cert, [#'Extension'{extnID = ?'id-ce-nameConstraints',
 				  extnValue = NameConst} | Rest],
 		    ValidationState,
 		    ExistBasicCon, SelfSigned, UserState, VerifyFun) ->
@@ -772,63 +799,69 @@ validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-nameConstraints',
     NewValidationState = add_name_constraints(Permitted, Excluded,
 					      ValidationState),
 
-    validate_extensions(OtpCert, Rest, NewValidationState, ExistBasicCon,
+    validate_extensions(Cert, Rest, NewValidationState, ExistBasicCon,
 			SelfSigned, UserState, VerifyFun);
-validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-certificatePolicies',
+validate_extensions(Cert, [#'Extension'{extnID = ?'id-ce-certificatePolicies',
 					   extnValue = Info}
 			      | Rest],
                     ValidationState,
 		    ExistBasicCon, SelfSigned, UserState, VerifyFun) ->
     Tree = process_policy_tree(Info, SelfSigned, ValidationState),
-    validate_extensions(OtpCert, Rest,
+    validate_extensions(Cert, Rest,
 			ValidationState#path_validation_state{
                           policy_ext_present = true,
                           current_any_policy_qualifiers =
                               current_any_policy_qualifiers(Info),
 			  valid_policy_tree = Tree},
 			ExistBasicCon, SelfSigned, UserState, VerifyFun);
-validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-policyConstraints'} = Ext
+validate_extensions(Cert, [#'Extension'{extnID = ?'id-ce-policyConstraints'} = Ext
 			      | Rest], ValidationState, ExistBasicCon,
 		    SelfSigned, UserState, VerifyFun) ->
     NewValidationState = ValidationState#path_validation_state{policy_constraint_ext = Ext},
-    validate_extensions(OtpCert, Rest, NewValidationState, ExistBasicCon,
+    validate_extensions(Cert, Rest, NewValidationState, ExistBasicCon,
 			SelfSigned, UserState, VerifyFun);
-validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-policyMappings'} = Ext
+validate_extensions(Cert, [#'Extension'{extnID = ?'id-ce-policyMappings'} = Ext
                              | Rest], ValidationState, ExistBasicCon,
 		    SelfSigned, UserState, VerifyFun) ->
     NewValidationState = ValidationState#path_validation_state{policy_mapping_ext = Ext},
-    validate_extensions(OtpCert, Rest, NewValidationState, ExistBasicCon,
+    validate_extensions(Cert, Rest, NewValidationState, ExistBasicCon,
 			SelfSigned, UserState, VerifyFun);
-validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-inhibitAnyPolicy'} = Ext
+validate_extensions(Cert, [#'Extension'{extnID = ?'id-ce-inhibitAnyPolicy'} = Ext
 			      | Rest], ValidationState, ExistBasicCon,
 		    SelfSigned, UserState, VerifyFun) ->
     NewValidationState = ValidationState#path_validation_state{policy_inhibitany_ext = Ext},
-    validate_extensions(OtpCert, Rest, NewValidationState, ExistBasicCon,
+    validate_extensions(Cert, Rest, NewValidationState, ExistBasicCon,
 			SelfSigned, UserState, VerifyFun);
-validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-extKeyUsage',
+validate_extensions(Cert, [#'Extension'{extnID = ?'id-ce-extKeyUsage',
                                            critical = true,
-                                           extnValue = KeyUse} = Extension | Rest],
+                                           extnValue = ExtKeyUse} = Extension | Rest],
 		    #path_validation_state{last_cert = false} = ValidationState, ExistBasicCon,
 		    SelfSigned, UserState0, VerifyFun) ->
     UserState =
-        case ext_keyusage_includes_any(KeyUse) of
+        case ext_keyusage_includes_any(ExtKeyUse) of
             true -> %% CA cert that specifies ?anyExtendedKeyUsage should not be marked critical
-                verify_fun(OtpCert, {bad_cert, invalid_ext_key_usage}, UserState0, VerifyFun);
+                verify_fun(Cert, {bad_cert, invalid_ext_key_usage}, UserState0, VerifyFun);
             false ->
-                verify_fun(OtpCert, {extension, Extension}, UserState0, VerifyFun)
+                case ca_known_extend_key_use(ExtKeyUse) of
+                    true ->
+                        UserState0;
+                    false ->
+                        verify_fun(Cert, {extension, Extension}, UserState0, VerifyFun)
+                end
         end,
-    validate_extensions(OtpCert, Rest, ValidationState, ExistBasicCon, SelfSigned,
+    validate_extensions(Cert, Rest, ValidationState, ExistBasicCon, SelfSigned,
 			UserState, VerifyFun);
-validate_extensions(OtpCert, [#'Extension'{} = Extension | Rest],
+validate_extensions(Cert, [#'Extension'{} = Extension | Rest],
 		    ValidationState, ExistBasicCon,
 		    SelfSigned, UserState0, VerifyFun) ->
-    UserState = verify_fun(OtpCert, {extension, Extension}, UserState0, VerifyFun),
-    validate_extensions(OtpCert, Rest, ValidationState, ExistBasicCon, SelfSigned,
+    UserState = verify_fun(Cert, {extension, Extension}, UserState0, VerifyFun),
+    validate_extensions(Cert, Rest, ValidationState, ExistBasicCon, SelfSigned,
 			UserState, VerifyFun).
 
-handle_last_cert(OtpCert, #path_validation_state{last_cert = true,
+handle_last_cert(Cert, #path_validation_state{last_cert = true,
                                                  user_initial_policy_set = PolicySet,
                                                  valid_policy_tree = Tree} = ValidationState0) ->
+    OtpCert = otp_cert(Cert),
     TBSCert = OtpCert#'OTPCertificate'.tbsCertificate,
     Extensions =
         extensions_list(TBSCert#'OTPTBSCertificate'.extensions),
@@ -843,11 +876,23 @@ handle_last_cert(OtpCert, #path_validation_state{last_cert = true,
                 ValidationState0
     end,
     ValidTree = policy_tree_intersection(PolicySet, Tree),
-    validate_policy_tree(OtpCert,
+    validate_policy_tree(Cert,
                          ValidationState#path_validation_state{valid_policy_tree = ValidTree});
 handle_last_cert(_, ValidationState) ->
     ValidationState.
 
+validate_ext_key_usage(#cert{otp = OtpCert} = Cert, UserState, VerifyFun, Type) ->
+    TBSCert = OtpCert#'OTPCertificate'.tbsCertificate,
+    Extensions = extensions_list(TBSCert#'OTPTBSCertificate'.extensions),
+    KeyUseExt = pubkey_cert:select_extension(?'id-ce-keyUsage', Extensions),
+    ExtKeyUseExt =  pubkey_cert:select_extension(?'id-ce-extKeyUsage', Extensions),
+    case compatible_ext_key_usage(KeyUseExt, ExtKeyUseExt, Type) of
+        true ->
+            UserState;
+        false ->
+            verify_fun(Cert, {bad_cert, {key_usage_mismatch, {KeyUseExt, ExtKeyUseExt}}},
+                       UserState, VerifyFun)
+    end.
 
 %%====================================================================
 %% Policy handling
@@ -1024,13 +1069,13 @@ any_ext_policy_children(#{expected_policy_set := ExpPolicySet}, Qualifiers, AllL
 %% Start Prepare Next Cert Policy Handling  RFC 5280 Section 6.1.4 -------------
 
 %% 6.1.4. b start:
-handle_policy_mappings(OtpCert,
+handle_policy_mappings(Cert,
                        #path_validation_state{valid_policy_tree = Tree0,
                                               policy_mapping_ext =
                                                   #'Extension'{extnID = ?'id-ce-policyMappings',
                                                                extnValue = PolicyMappings}}
                        = ValidationState) ->
-    case handle_policy_mappings(PolicyMappings, OtpCert, Tree0, ValidationState) of
+    case handle_policy_mappings(PolicyMappings, Cert, Tree0, ValidationState) of
         {tree, Tree} ->
             ValidationState#path_validation_state{valid_policy_tree = Tree};
         {user_state, UState} ->
@@ -1039,10 +1084,10 @@ handle_policy_mappings(OtpCert,
 
 handle_policy_mappings([], _, Tree, _) ->
     {tree, Tree};
-handle_policy_mappings([Mappings | Rest], OtpCert, Tree0, ValidationState) ->
-    case handle_policy_mapping(Mappings, OtpCert, Tree0, ValidationState) of
+handle_policy_mappings([Mappings | Rest], Cert, Tree0, ValidationState) ->
+    case handle_policy_mapping(Mappings, Cert, Tree0, ValidationState) of
         {tree, Tree} ->
-            handle_policy_mappings(Rest, OtpCert, Tree, ValidationState);
+            handle_policy_mappings(Rest, Cert, Tree, ValidationState);
         Other ->
             Other
     end.
@@ -1055,7 +1100,7 @@ handle_policy_mapping(#'PolicyMappings_SEQOF'{
                              IssuerPolicy,
                          subjectDomainPolicy =
                              SubjectPolicy} = Ext,
-                      OtpCert, Tree0,
+                      Cert, Tree0,
                       #path_validation_state{inhibit_policy_mapping =
                                                  PolicyMappingConstraint,
                                              current_any_policy_qualifiers =
@@ -1070,7 +1115,7 @@ handle_policy_mapping(#'PolicyMappings_SEQOF'{
                                              PolicyMappingConstraint, AnyQualifiers),
             {tree, Tree};
         false ->
-            UserState = verify_fun(OtpCert, {bad_cert, {invalid_policy_mapping, Ext}},
+            UserState = verify_fun(Cert, {bad_cert, {invalid_policy_mapping, Ext}},
                                    UserState, VerifyFun),
             {user_state, UserState}
     end.
@@ -1737,8 +1782,8 @@ is_fixed_dh_cert(PublicKeyInfo, Extensions) ->
     case select_extension(?'id-ce-keyUsage', Extensions) of
 	undefined ->
 	    is_dh(Algorithm);
-	#'Extension'{extnValue=KeyUse} ->
-	    is_dh(Algorithm) andalso is_valid_key_usage(KeyUse, keyAgreement)
+	#'Extension'{extnValue = KeyUses} ->
+	    is_dh(Algorithm) andalso lists:member(keyAgreement, KeyUses)
     end.
 	
 is_dh(?'dhpublicnumber')->
@@ -1746,7 +1791,8 @@ is_dh(?'dhpublicnumber')->
 is_dh(_) ->
     false.
 
-is_digitally_sign_cert(OtpCert) ->
+is_digitally_sign_cert(Cert) ->
+    OtpCert = otp_cert(Cert),
     TBSCert = OtpCert#'OTPCertificate'.tbsCertificate,
     Extensions = extensions_list(TBSCert#'OTPTBSCertificate'.extensions),
     case pubkey_cert:select_extension(?'id-ce-keyUsage', Extensions) of
@@ -1756,6 +1802,79 @@ is_digitally_sign_cert(OtpCert) ->
 	    lists:member(keyCertSign, KeyUse)
     end.
 
+compatible_ext_key_usage(undefined, _, endentity) ->
+    true;
+compatible_ext_key_usage(_, undefined, _) ->
+    true;
+compatible_ext_key_usage(#'Extension'{extnID = ?'id-ce-keyUsage',
+                                      extnValue = KeyUses},
+                         #'Extension'{extnID = ?'id-ce-extKeyUsage',
+                                      extnValue = Purposes}, Type) ->
+    case ext_keyusage_includes_any(Purposes) of
+        true ->
+            true;
+        false ->
+            is_compatible_purposes(KeyUses, Purposes, Type)
+    end.
+
+is_compatible_purposes(_, [], _) ->
+    true;
+is_compatible_purposes(KeyUses, [?'id-kp-serverAuth'| Rest], ca = Type) ->
+    %% keyCertSign is already verified for a ca and considered compatible
+    is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [?'id-kp-serverAuth'| Rest], endentity = Type) ->
+    IsServerAuthComp = case lists:member(digitalSignature, KeyUses) of
+                           true ->
+                               true;
+                           false ->
+                               lists:member(keyAgreement, KeyUses)  orelse
+                                   lists:member(keyEncipherment, KeyUses)
+                       end,
+    IsServerAuthComp andalso is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [?'id-kp-clientAuth'| Rest], ca = Type) ->
+    %% keyCertSign is already verified for a ca and considered compatible
+    is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [?'id-kp-clientAuth'| Rest], endentity = Type) ->
+    IsClientAuthComp = case lists:member(digitalSignature, KeyUses) of
+                           true ->
+                               true;
+                           false ->
+                               lists:member(keyAgreement, KeyUses)
+                       end,
+    IsClientAuthComp andalso is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [?'id-kp-codeSigning'| Rest], Type) ->
+    lists:member(digitalSignature, KeyUses) andalso
+        is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [?'id-kp-emailProtection'| Rest], Type) ->
+    IsEmailProtCompatible = case (lists:member(digitalSignature, KeyUses) orelse
+                                  lists:member(nonRepudiation, KeyUses)) of
+                                true ->
+                                    true;
+                                false ->
+                                    lists:member(keyAgreement, KeyUses) orelse
+                                        lists:member(keyEncipherment, KeyUses)
+                            end,
+    IsEmailProtCompatible andalso is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [Id| Rest],Type) when Id == ?'id-kp-timeStamping';
+                                                      Id == ?'id-kp-OCSPSigning'->
+    (lists:member(digitalSignature, KeyUses) orelse
+     lists:member(nonRepudiation, KeyUses)) andalso
+        is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [_| Rest], Type) -> %% Unknown purposes are for user verify_fun to care about
+    is_compatible_purposes(KeyUses, Rest, Type).
+
+
+ca_known_extend_key_use(ExtKeyUses) ->
+    CAExtSet = ca_known_ext_key_usage(),
+    Intersection = sets:intersection(CAExtSet, sets:from_list(ExtKeyUses)),
+    not sets:is_empty(Intersection).
+
+ca_known_ext_key_usage() ->
+    %% Following extended key usages are known
+    sets:from_list([?'id-kp-serverAuth', ?'id-kp-clientAuth',
+                    ?'id-kp-codeSigning', ?'id-kp-emailProtection',
+                    ?'id-kp-timeStamping', ?'id-kp-OCSPSigning']).
+
 missing_basic_constraints(OtpCert, SelfSigned, ValidationState, VerifyFun, UserState0,Len) ->
     UserState = verify_fun(OtpCert, {bad_cert, missing_basic_constraint},
 			   UserState0, VerifyFun),
@@ -1768,9 +1887,6 @@ missing_basic_constraints(OtpCert, SelfSigned, ValidationState, VerifyFun, UserS
 	     UserState}
     end.
 
-is_valid_key_usage(KeyUse, Use) ->
-    lists:member(Use, KeyUse).
-
 %%====================================================================
 %% Generate test data
 %%====================================================================
@@ -2012,7 +2128,7 @@ extensions(Role, Type, Opts) ->
 
 add_default_extensions(_, ca, Exts) ->
     Default = [#'Extension'{extnID = ?'id-ce-keyUsage',
-                            extnValue = [keyCertSign, cRLSign],
+                            extnValue = [keyCertSign, digitalSignature, cRLSign],
                             critical = false},
                #'Extension'{extnID = ?'id-ce-basicConstraints',
                             extnValue = #'BasicConstraints'{cA = true},
@@ -2029,9 +2145,12 @@ add_default_extensions(server, peer, Exts) ->
                             critical = false}
               ],
     add_default_extensions(Default, Exts);
-
 add_default_extensions(client, peer, Exts) ->
-    Exts.
+    Default = [#'Extension'{extnID = ?'id-ce-keyUsage',
+                            extnValue = [digitalSignature],
+                            critical = false}
+              ],
+    add_default_extensions(Default, Exts).
 
 add_default_extensions(Defaults0, Exts) ->
     Defaults = lists:filtermap(fun(#'Extension'{extnID = ID} = Ext) ->
@@ -2065,8 +2184,27 @@ verify_options(
      {rsa_pss_saltlen, SaltLen},
      {rsa_mgf1_md, HashAlgo}].
 
-ext_keyusage_includes_any(KeyUse) when is_list(KeyUse) ->
-    lists:member(?anyExtendedKeyUsage, KeyUse);
+ext_keyusage_includes_any(KeyUses) when is_list(KeyUses) ->
+    lists:member(?anyExtendedKeyUsage, KeyUses);
 ext_keyusage_includes_any(_) ->
     false.
 
+otp_cert(Der) when is_binary(Der) ->
+    public_key:pkix_decode_cert(Der, otp);
+otp_cert(#'OTPCertificate'{} = Cert) ->
+    Cert;
+otp_cert(#cert{otp = OtpCert}) ->
+    OtpCert.
+
+key_params(#'OTPTBSCertificate'{signature =
+                                    #'SignatureAlgorithm'{algorithm =
+                                                              ?'id-RSASSA-PSS',
+                                                          parameters = KeyParams}},
+           KeyParams0) when KeyParams0 == asn1_NOVALUE;
+                            KeyParams0 == 'NULL' ->
+    %% Sometimes parameters may be missing in issuer's
+    %% "SubjectPublicKeyInfo" but included in the certs
+    %% "SignatureAlgorithm" for RSA PSS signatures.
+    KeyParams;
+key_params(_, KeyParams) ->
+    KeyParams.
